//*******  Project: Legend of Zelda Treasure Chest  *******
//*******  Author: Kevin Martinez                   *******
//*******  Date Last Modified: November 14, 2015    *******


#include "RupeeEvent.h"
#include "Wire.h"
#include "LiquidCrystal_I2C.h"


//Defining which pins will be responsible for a given component
byte redPin = 9;
byte greenPin = 10;
byte bluePin = 11;
byte ledPin = 6;
byte mainLid = 7;



//Constructor for an event
RupeeEvent::RupeeEvent(int _color, LiquidCrystal_I2C* _disp)
{
  myDisplay = _disp;
  color = _color;

  //Defining the frequencies that pins 9, 10, & 11 will output to display the color
  //Also defining the three messages received by the user
  if(color == 1)
  {
    redFreq = 0;
    greenFreq = 255;
    blueFreq = 0;
    text1 = "You got a Green Rupee!";
    text2 = "That's worth oneRupee!";
    text3 = "Keep it in your wallet for now.";
  }
  if(color == 2)
  {
    redFreq = 0;
    greenFreq = 0;
    blueFreq = 255;
    text1 = "You got a Blue  Rupee!";
    text2 = "That's worth 5  Rupees!";
    text3 = "Not too shabby! ";
  }
  if(color == 3)
  {
    redFreq = 255;
    greenFreq = 255;
    blueFreq = 0;
    text1 = "You got a YellowRupee!";
    text2 = "That's worth 10 Rupees!";
    text3 = "What a pleasant surprise!";
  }
  if(color == 4)
  {
    redFreq = 255;
    greenFreq = 0;
    blueFreq = 0;
    text1 = "You got a Red   Rupee!";
    text2 = "That's worth 20 Rupees!";
    text3 = "Don't spend it  all in 1 place!";
  }
  if(color == 5)
  {
    redFreq = 255;
    greenFreq = 0;
    blueFreq = 255;
    text1 = "You got a PurpleRupee!";
    text2 = "That's worth 50 Rupees!";
    text3 = "Your face is    beaming!";
  }
  if(color == 6)
  {
    redFreq = 255;
    greenFreq = 55;
    blueFreq = 0;
    text1 = "You got a OrangeRupee!";
    text2 = "That's worth 100Rupees!";
    text3 = "You feel like   a celebrity!";
  }
  if(color == 7)
  {
    redFreq = 255;
    greenFreq = 255;
    blueFreq = 255;
    text1 =  "You got a SilverRupee!";
    text2 =  "That's worth 200Rupees!";
    text3 =  "Wallet's full.  Lets put it back";
  }
}



// Flashes yellow LED's as song builds
void RupeeEvent::flashLEDs()
{ 

//Loop that raises and lower intensity of LED's
//Ten flashes that get faster with each iteration
for(int j=0; j<9; j++)
    {  

      for (int i=255; i>0; i--)
      {
        analogWrite(ledPin, i);
        delayMicroseconds(800-60*j);
      }
      delay(200-j*15); 



      for (int i=0; i<255; i++)
      {
        analogWrite(ledPin, i);
        delayMicroseconds(800-60*j);
      } 
      delay(200-j*15);  
    }
}



//Smoothly sets a color to rupee
void RupeeEvent::setColor()
{ 
  //Sets voltage to 5V
  digitalWrite(8, HIGH);
  
  for (int i=255; i>0; i--)
  {
    if(i<redFreq)
    analogWrite(redPin, i);

    if(color==6)  //Special condition for orange rupee
    analogWrite(greenPin, ceil(200+i*greenFreq/255));
    else if(i<greenFreq)  //For all other colors
    analogWrite(greenPin, i);
    
    if(i<blueFreq)
    analogWrite(bluePin, i);

    delay(5);
  }
}

void RupeeEvent::playSound(String sound)
{
  if(sound=="unlock")  Serial.write(1);
  if(sound=="lock")  Serial.write(2);
  if(sound=="open")  Serial.write(3);
  if(sound=="secret")  Serial.write(4);
  if(sound=="grab1")  Serial.write(5);
  if(sound=="grab2")  Serial.write(6);
  if(sound=="get")  Serial.write(7);
  if(sound=="count")  {Serial.write(111); Serial.write(8);}
  if(sound=="rupee")  {Serial.write(111);Serial.write(9);}
  if(sound=="letter") {Serial.write(111); Serial.write(10);}
  if(sound=="sentence") {Serial.write(111);Serial.write(11);}
  if(sound=="sweep")  Serial.write(12);
}


//Activates LCD screen and displays three messages
//Shimmers LED until event is ended
void RupeeEvent::displayText()
{ 
    //Turns on LCD backlight
    myDisplay->backlight();

      //Prints out first message letter by letter
      for(int j=0; j<16; j++)
      { 
        if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        myDisplay->setCursor(j,0);
        myDisplay->print(text1.charAt(j));
        playSound("letter");
        delay(100);}
      }
      
      for(int j=16; j<text1.length(); j++)
      {
        if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        myDisplay->setCursor(j-16,1);
        myDisplay->print(text1.charAt(j));
        playSound("letter");
        delay(100);}
      }
      if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        playSound("sentence");
        delay(2000);
        myDisplay->clear();}

      //Prints out second message letter by letter
      for(int j=0; j<16; j++)
      { 
        if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        myDisplay->setCursor(j,0);
        myDisplay->print(text2.charAt(j));
        playSound("letter");
        delay(100);}
      }
      for(int j=16; j<text2.length(); j++)
      {
        if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        myDisplay->setCursor(j-16,1);
        myDisplay->print(text2.charAt(j));
        playSound("letter");
        delay(100);}
      }
      if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        playSound("sentence");
        delay(2000); 
        myDisplay->clear();}

      //Prints out third message letter by letter
      for(int j=0; j<16; j++)
      { 
        if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        myDisplay->setCursor(j,0);
        myDisplay->print(text3.charAt(j));
        playSound("letter");
        delay(100);}
      }
      for(int j=16; j<text3.length(); j++)
      {
        if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        myDisplay->setCursor(j-16,1);
        myDisplay->print(text3.charAt(j));
        playSound("letter");
        delay(100);}
      }
      if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        playSound("sentence");
        delay(2000); 
        myDisplay->clear();}

      if(color==7)
      {
      String text4 = "Just kidding!   Let's keep it!  ";
      
      for(int j=0; j<16; j++)
      { 
        if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        myDisplay->setCursor(j,0);
        myDisplay->print(text4.charAt(j));
        playSound("letter");
        delay(100);}
      }
      for(int j=16; j<text3.length(); j++)
      {
        if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        myDisplay->setCursor(j-16,1);
        myDisplay->print(text4.charAt(j));
        playSound("letter");
        delay(100);}
      }
      if(analogRead(A0)<700 && digitalRead(mainLid)==HIGH){
        playSound("sentence");
        delay(2000); 
        myDisplay->clear();}

      }
}


void RupeeEvent::shimmer()
{
  int incomingByte= 0; 
  
        for (int i=0; i<=155; i++)
        {
            if(i<redFreq)
            analogWrite(redPin, i);
        
            if(color==6)
            analogWrite(greenPin, ceil(200+i*greenFreq/255));
            else if(i<greenFreq)
            analogWrite(greenPin, i);
        
            if(i<blueFreq)
            analogWrite(bluePin, i);
            delay(5);

            if (Serial.available()>0){
             incomingByte = Serial.read();
            }

            if(incomingByte == 1) changeColor(color+1);
          }

          changeColor(color+1);
          
          for (int i=155; i>0; i--)
          {
            if(i<redFreq)
            analogWrite(redPin, i);
        
            if(color==6)
            analogWrite(greenPin, ceil(200+i*greenFreq/255));
            else if(i<greenFreq)
            analogWrite(greenPin, i);
            
            if(i<blueFreq)
            analogWrite(bluePin, i);
            delay(5);

            if (Serial.available()>0){
             incomingByte = Serial.read();
            }

            if(incomingByte == 1) changeColor(color+1);
          }
      
}

void RupeeEvent::changeColor(int _color)
{
  color = _color;

  if(_color == 1)
  {
    redFreq = 0;
    greenFreq = 255;
    blueFreq = 0;
  }
  if(_color == 2)
  {
    redFreq = 0;
    greenFreq = 0;
    blueFreq = 255;
  }
  if(_color == 3)
  {
    redFreq = 255;
    greenFreq = 255;
    blueFreq = 0;
  }
  if(_color == 4)
  {
    redFreq = 255;
    greenFreq = 0;
    blueFreq = 0;
  }
  if(_color == 5)
  {
    redFreq = 255;
    greenFreq = 0;
    blueFreq = 255;
  }
  if(color == 6)
  {
    redFreq = 255;
    greenFreq = 55;
    blueFreq = 0;
  }
  if(color == 7)
  {
    redFreq = 255;
    greenFreq = 255;
    blueFreq = 255;
  }

  if(color == 8)
  {
    color = 1;
    redFreq = 0;
    greenFreq = 255;
    blueFreq = 0;
  }
}


//Function called in main loop to end the sequence
void RupeeEvent::endEvent()
{   
    myDisplay->clear();
    myDisplay->noBacklight();
    
    //Smoothly turns off rupee
    for (int i=0; i<=255; i++)
    {
      if(i<redFreq)
      analogWrite(redPin, i);
  
      if(color==6)
      analogWrite(greenPin, ceil(200+i*greenFreq/255));
      else if(i<greenFreq)
      analogWrite(greenPin, i);
  
      if(i<blueFreq)
      analogWrite(bluePin, i);
  
      delay(5);
    }

    delay(200);
    digitalWrite(8, LOW);  //Sets RGB LED to 0V

    Serial.write(111);
}



