///*******  Project: Legend of Zelda Treasure Chest  *******
//*******  Author: Kevin Martinez                   *******
//*******  Date Last Modified: November 14, 2015    *******


#include "Wire.h"
#include "LiquidCrystal_I2C.h"
#include "RupeeEvent.h"

#define interruptNumber 0
#define secretLid 2
#define mainLid 7

//Creating LCD object
LiquidCrystal_I2C myDisplay(0x27,16,2);

//Variable that decides which color is displayed
byte event;
byte prevEvent;

byte rupeeCount = 0;
byte prevRupees = 0;

byte rupeeDef[8] = {
B00100,
B01110,
B11111,
B11111,
B11111,
B11111,
B01110,
B00100
};

const byte rupee = 0x0;

void setup() {
Serial.begin(9600);
  
//Initializing pins for lid and RGB LED 5V lead
pinMode(mainLid, INPUT);
digitalWrite(mainLid, HIGH);
pinMode(8, OUTPUT);
pinMode(secretLid, INPUT);
digitalWrite(secretLid, HIGH);

myDisplay.init();   //Initializes LCD screen

myDisplay.createChar(0, rupeeDef);

//Sets first event
randomSeed(1);
event = random(1,8);

attachInterrupt(interruptNumber, InterruptServiceRoutine, RISING);
}

void loop() {
  
    prevEvent = event;   //Keeps track of last event
    prevRupees = rupeeCount;

    RupeeEvent myEvent(event, &myDisplay);

   if(analogRead(A0)>700)
   {
    while(analogRead(A0)>700); 
    myEvent.playSound("unlock");   //Plays unlock sound when potentiometer is deflected
   }

    while(analogRead(A0)<700 && digitalRead(mainLid)==HIGH)
    {
      delay(500);
      myEvent.playSound("open");   //Starts jingle
      delay(500);
      myEvent.flashLEDs();   //Blink LED's while song is building up
      
      delay(1500);   //Small pause

      //Depending on the tier of the prize, different sound is played
      if(event==1 || event==2 || event==3) myEvent.playSound("grab1");
      else if(event==4 || event==5) myEvent.playSound("grab2");
      else if(event==6 || event==7) myEvent.playSound("get");
      
      myEvent.setColor();   //Color of rupee is revealed
  
      delay(1500); if(event==7) delay(1000);
      
      myEvent.displayText();   //Messages are displayed, RGB LED

      if(event==1) rupeeCount = prevRupees + 1;
      if(event==2) rupeeCount = prevRupees + 5;
      if(event==3) rupeeCount = prevRupees + 10;
      if(event==4) rupeeCount = prevRupees + 20;
      if(event==5) rupeeCount = prevRupees + 50;
      if(event==6) rupeeCount = prevRupees + 100;
      if(event==7) rupeeCount = prevRupees + 200;

      myDisplay.setCursor(6,0);
      myDisplay.write(0);
      myDisplay.print(" ");
      myDisplay.print(prevRupees);
      delay(1000);
      myDisplay.clear();
      
      for(int i= 0; i<=rupeeCount-prevRupees; i++){
        myDisplay.setCursor(6,0);
        myDisplay.write(0);
        myDisplay.print(" ");
        myDisplay.print(prevRupees+i);
        if(event != 1) myEvent.playSound("count");
        if(i<rupeeCount-prevRupees) delay(50);
        myDisplay.clear();
      }

      myEvent.playSound("rupee");

      myDisplay.setCursor(6,0);
      myDisplay.write(0);
      myDisplay.print(" ");
      myDisplay.print(rupeeCount);

      delay(2000);

      myEvent.playSound("sweep");

      while(analogRead(A0)<700 && digitalRead(mainLid)==HIGH)
      {          
            myEvent.shimmer();
      }
   }

    Serial.write(111);
    delay(99);
    Serial.write(112);
   
    if(analogRead(A0)>700) myEvent.playSound("lock");

    myEvent.endEvent();

    do{event = random(1,8);} while(event==prevEvent); //Do-while loop to ensure the next event selected is distinct from the previous
}

void InterruptServiceRoutine()
{ 
  Serial.write(111);
  delay(99);
  Serial.write(112);
  Serial.write(4);
}

